#include <built_in.h>
#include <stdlib.h>

#define PIC18F452
short CPUClock = 40; // from a 10 MHz crystal and the PIC 4x PLL

// Lcd module connections as defined by MikroC
sbit LCD_RS at RB0_bit;
sbit LCD_EN at RB1_bit;
sbit LCD_D4 at RB2_bit;
sbit LCD_D5 at RB3_bit;
sbit LCD_D6 at RB4_bit;
sbit LCD_D7 at RB5_bit;

sbit LCD_RS_Direction at TRISB0_bit;
sbit LCD_EN_Direction at TRISB1_bit;
sbit LCD_D4_Direction at TRISB2_bit;
sbit LCD_D5_Direction at TRISB3_bit;
sbit LCD_D6_Direction at TRISB4_bit;
sbit LCD_D7_Direction at TRISB5_bit;


sbit HalfStep at RC7_bit;
sbit Ref250 at RA4_bit;

// Define Data Bus bits: DM0...DM7
sbit RM0 at RD0_bit;
sbit RM1 at RD1_bit;
sbit RM2 at RD2_bit;
sbit RM3 at RD3_bit;
sbit RM4 at RD4_bit;
sbit RM5 at RD5_bit;
sbit RM6 at RD6_bit;
sbit RM7 at RD7_bit;

// A/D Inputs
sbit FP0 at RA0_bit;
sbit FP1 at RA1_bit;
sbit FP2 at RA2_bit;
sbit FP3 at RA3_bit;
sbit FP4 at RA5_bit;
sbit FP5 at RE0_bit;
sbit FP6 at RE1_bit;
sbit FP7 at RE2_bit;

// U3TC is used as the Freq Counter signal input
 sbit SIGIN at RC0_bit;
 
  // Clock Generator clock source
 sbit SELINA at RC1_bit;
 sbit SELINB at RC2_bit;
 
  // A clock pin under software control used to store into the
 sbit PICCLK at RC3_bit;

 // 74579 Master Reset
 sbit U3MR_ at RC6_bit;

// RAM Control bits
 sbit U4OE_ at RC4_bit;
 sbit U4WE_ at RC5_bit;

 #define PICDirectMode 11
 #define FunctionMode 10
 #define FreqCntrMode1 01
 #define FreqCntrMode4 00

 #define FCMode 1
 #define VVRMode 2
 #define SineMode 3
 #define SquareMode 4
 #define RampMode 5
 #define SawMode 6
 #define PulseMode 7
 #define F1Mode 8
 #define F2Mode 9
 #define F3Mode 10
 
int                      FunctionType;
unsigned int             T1OverflowCount;
unsigned int             T2OverflowCount;
int                      CurrentMode, CurrentBits;
unsigned char            LCDString[21];
unsigned long            ulFrequency;
unsigned long            ulTotalPeriod;
float                    Frequency;
unsigned long            i, j;
float                    fFrequency, fTotalPeriod, OutPutVolts, SwitchReturn;
int                      SelectedMode, SelectedBits;
unsigned int             OverFlowCount, Resolution, Mhz;
unsigned int             ADResult;
int                      PORTDSet, k, GateTime, Timer2First, Timer2Second;
int                       Count1,PR2Set, PR2Set2, PR2Set3, FreqMultiplier, PR2Trim;


void WriteLCD(int Line, int Start, char* string)
{

  Lcd_Out(Line,Start,string);   // A mikroC built-in LCD routine

  return;
}


// The following routine reads the selected A/D converter

float SetAndReadAD (int ADPort, int LCDLine, int LCDPosition)
  {
    switch (ADPort)
    {
       case 0:
         ADCON0.CHS0 = 0;
         ADCON0.CHS1 = 0;
         ADCON0.CHS2 = 0;
         break;
       case 1:
         ADCON0.CHS0 = 1;
         ADCON0.CHS1 = 0;
         ADCON0.CHS2 = 0;
         break;
       case 2:
         ADCON0.CHS0 = 0;
         ADCON0.CHS1 = 1;
         ADCON0.CHS2 = 0;
         break;
      case 3:
         ADCON0.CHS0 = 1;
         ADCON0.CHS1 = 1;
         ADCON0.CHS2 = 0;
         break;
      case 4:
         ADCON0.CHS0 = 0;
         ADCON0.CHS1 = 0;
         ADCON0.CHS2 = 1;
         break;
      case 5:
         ADCON0.CHS0 = 1;
         ADCON0.CHS1 = 0;
         ADCON0.CHS2 = 1;
         break;
      case 6:
         ADCON0.CHS0 = 0;
         ADCON0.CHS1 = 1;
         ADCON0.CHS2 = 1;
         break;
      case 7:
         ADCON0.CHS0 = 1;
         ADCON0.CHS1 = 1;
         ADCON0.CHS2 = 1;
         break;
    }
    delay_ms(100);  //TO CHARGE THE CAPACITOR!!
      ADCON0.GO = 1;

      while(ADCON0.GO)
      i=i;

      ADResult = ADRESL + (ADRESH << 8);
      Lcd_Cmd(_LCD_CLEAR);

  //      IntToStr(ADResult, LCDString);
   //     WriteLCD(1, 1, LCDString);

      if (ADPort == 1)
        OutPutVolts = (float)ADResult/(float)1023 * (float)10;
      else
        OutPutVolts = (float)ADResult/(float)1023 * (float)5;

      FloatToStr(OutPutVolts, LCDString);
      strcat(LCDString, "OV");

     if (LCDLine)
     {
        WriteLCD(LCDLine, LCDPosition, LCDString);
        WordToStr(ADResult, LCDString);
        WriteLCD(LCDLine, LCDPosition + 12, LCDString);
     }
      return(OutPutVolts);
  }

// The following routine scans the MODE switch SW1

int ReadModeSwitch()
{
  SwitchReturn = SetAndReadAD(4, 1, 1);   // Get the selected Mode Lcd_Cmd(_LCD_CLEAR);

   if(SwitchReturn < (float).24)   // FC volts
   {
     SelectedMode = FCMode;
   }
   else if(SwitchReturn < (float).75)   // VVR volt
   {
     SelectedMode = VVRMode;
   }
    else if(SwitchReturn < (float)1.4)  // Sine volts
   {
     SelectedMode = SineMode;
   }
    else if(SwitchReturn < (float)1.96)   // Square volts
   {
     SelectedMode = SquareMode;
   }
    else if(SwitchReturn < (float)2.50)   //Ramp volts
   {
     SelectedMode = RampMode;
   }
    else if(SwitchReturn < (float)3.05)    // Saw volts
   {
     SelectedMode = SawMode;
   }
    else if(SwitchReturn < (float)3.61)   // Pulse volts
   {
    SelectedMode = PulseMode;
   }
    else if(SwitchReturn < (float)4.15)    // F1 volts
   {
     SelectedMode = F1Mode;
   }
    else if(SwitchReturn < (float)4.72)     // F2 volts
   {
     SelectedMode = F2Mode;
   }
    else    // F3 volts
   {
    SelectedMode = F3Mode;
   }

   return(SelectedMode);
}

// The following routine scans the BITS switch SW4

char SelectBits ()
{
 SwitchReturn = SetAndReadAD(7, 1, 1) ;

  if(SwitchReturn < (float)1.5)   // 16 bits
   {
    CurrentBits = 16;
   }
   else if(SwitchReturn < (float)3.5)   // 32 bits
   {
     CurrentBits = 32;
   }
   else
     CurrentBits = 64;

   return (CurrentBits);
}

// The following routine generates a single pulse on PICCLK
// It's used as a clock to the 74F579 when the RAM is being filled

void OnePulse()
{
 PICCLK = 1;
 PICCLK = 0;
 PICCLK = 1;
}

//This routine writes data into the RAM

void WriteRAM()
{
 U4WE_ = 1;
 U4WE_ = 0;
 U4WE_ = 1;
}

//The following are the constants for 1/4 of a sine wave

unsigned char   SinWave64[] = {3,6,10,13,16,19,22,25,28,32,35,38,41,44,       //14
                               47,50,52,56,58,61,64,67,69,72,75,77,80,82,     //14
                               85,87,89,92,94,96,97,100,102,104,106,107,      //12
                               109,111,112,114,115,117,118,119,120,121,122,   //11
                               123,124,125,125,126,127,127,127,128,128,128,   //11
                               128,128};                                      ///2
                               
void DeleteLeadingSpaces(char * DisplayText)
 {
 k = 0;

  for (i=0; i<=strlen(DisplayText); i++)     // Delete leading spaces
  {
    if (DisplayText [i] != ' ')
    DisplayText[k++] = DisplayText[i];
  }
}

// The following routine makes a period measurement.
// CycleCount specifies the number of input cycles to count
// while Timer1 counts how many 10 Mhz cycles occur in CycleCount
// input cycles.

void ReadPeriod(unsigned long CycleCount)
{
  INTCON = 0;  // Disable Interrupts
  PIE1 = 0;
  PIR1 = 0;
  
  T1CON.TMR1ON = 0;
  PIR1.TMR1IF = 0;

  T1OverflowCount = 0;

  TMR1H = 0;
  TMR1L = 0;

  T1CON = 0b00000100;  // Timer1 set to 1:1 PreScale, Fosc/4 for input
  CycleCount += 1;

  while(SIGIN)          // Wait for the input to go low
   i=i;

    do
    {
      while(!SIGIN)          // Wait for the input to go high
      {
        if (PIR1.TMR1IF)   // Keep checking for timer1 overflow
        {
          T1OverflowCount++;
          PIR1.TMR1IF = 0;
          
          if (T1OverflowCount > 15000000)  // This a 1.5 second timeout that
            goto PeriodTimeout;            // stops the period measurement
        }
      }      // A low to high edge just ocurred

      T1CON.TMR1ON = 1;    // It does't matter if it's already on
      
      if (!(--CycleCount))  // See if we're done
        T1CON.TMR1ON = 0;   // Stop the timer
    
      while(SIGIN)          // Now wait for the input to go low
      {
        if (PIR1.TMR1IF)   // Keep checking for timer1 overflow
        {
          T1OverflowCount++;
          PIR1.TMR1IF = 0;
        }
        
        if (T1OverflowCount > 300)
            goto PeriodTimeout;
      }
    }
    while(CycleCount);
    
    ulTotalPeriod =  (TMR1H << 8) + TMR1L + (unsigned long)T1OverflowCount * 0xffff;
    ulTotalPeriod += 60;     // Correction for delays caused by above code
    
    fFrequency = ((float)10000000 /(float)ulTotalPeriod) * (float)ulFrequency;
    
    if (CurrentMode == FunctionMode)
      fFrequency = fFrequency / (float)256;
    else if (CurrentMode == FreqCntrMode4)
      fFrequency = fFrequency * 4;
    
    PeriodTimeout:
      if (T1OverflowCount > 15000000)
        ulTotalPeriod = 0;
}

// The following routine sets a gate time amd makes a frequency measurement.
// It's for gate times of 0.1 sec,, 0.5 sec. or 1 sec.
// times are for a 40 Mhz internat PIC clock based on a 10 Mhx crystal
// and the internal 4x phase locked loop circuit
// Timer 2 is used to set the gate time and Timer 1 is used to count the
// pulses

void ReadFrequency(short GateTime)
 {
   if (CPUClock == 40)
   {
     if (GateTime == 1)        // 1/10 second
     {
       Count1 = 244;
       PR2Set = 133;
       PR2Set2 = 1;
       PR2Set3 = 1;
       FreqMultiplier = 1;
     }
     else if (GateTime == 5)        // 1/2 second
     {
       Count1 = 1220;
       PR2Set = 255;
       PR2Set2 = 255;
       PR2Set3 = 214;
       FreqMultiplier = 2;
     }
     else if (GateTime == 10)        // 1 second
     {
       Count1 = 2441;
       PR2Set = 255;
       PR2Set2 = 183;
       PR2Set3 = 1;
       FreqMultiplier = 1;
     }
    }

  INTCON = 0;  // Disable Interrupts
  PIE1 = 0;
  PIR1 = 0;

   T2CON.TMR2ON = 0;
   T1CON.TMR1ON = 0;

   PIR1.TMR1IF = 0;
   PIR1.TMR2IF = 0;
   T1OverflowCount = 0;
   T2OverflowCount = 0;

   TMR1H = 0;
   TMR1L = 0;
   TMR2 = 0;
   PR2= 0xFF;

   T2CON = 0b00000110;  // Timer2 On and set to 16:1 PreScale
   T1CON = 0b00000111;  // Timer1 set to 1:1 PreScale
   PICCLK = 1;
   
TimerOverflow1:
    if (PIR1.TMR1IF)
    {
      T1OverflowCount++;
      PIR1.TMR1IF = 0;
    }

    if (!PIR1.TMR2IF)
      goto TimerOverflow1;

    PIR1.TMR2IF = 0;
    T2OverflowCount++;

 if (T2OverflowCount < Count1)
   goto TimerOverflow1;

   PIR1.TMR2IF = 0;

   PR2 = PR2Set;
   TMR2 = 0;
   T2CON = 0b00000101; // Set Timer 2 to 4:1         //

TimerOverflow2:
   if (!PIR1.TMR2IF)
      goto TimerOverflow2;

    if (PIR1.TMR1IF)
    {
      T1OverflowCount++;
      PIR1.TMR1IF = 0;
    }

    PIR1.TMR2IF = 0;
    PR2 = PR2Set2;
    TMR2 = 0;

  TimerOverflow3:
   if (!PIR1.TMR2IF)
      goto TimerOverflow3;

    if (PIR1.TMR1IF)
    {
      T1OverflowCount++;
      PIR1.TMR1IF = 0;
    }
    
    PIR1.TMR2IF = 0;
    PR2 = PR2Set3;
    TMR2 = 0;
    
    TimerOverflow4:
   if (!PIR1.TMR2IF)
      goto TimerOverflow4;

    if (PIR1.TMR1IF)
    {
      T1OverflowCount++;
      PIR1.TMR1IF = 0;
  }

    T1CON.TMR1ON = 0;
    TMR2 = 0;
    PICCLK = 0;

    ulFrequency =  (TMR1H << 8) + TMR1L + (unsigned long)T1OverflowCount * 0xffff ;
    ulFrequency = ulFrequency * FreqMultiplier;
      
    if (CurrentMode == FunctionMode)
      ulFrequency = ulFrequency >> 8;
    else if (CurrentMode == FreqCntrMode4)
      ulFrequency = ulFrequency << 2  ;
  }
  
  // This funtion sets the clock source for the 74F579
  
   void  SetInputMode(int Mode)
  {
    switch (Mode)
    {
      case PICDirectMode:    // PICCLK ib the clock source for the 74F579
        SELINA = 1;
        SELINB = 1;
        CurrentMode = PICDirectMode;
        break;
      case FunctionMode:      // Select HS Clock for Sine, Square etc.
        SELINA = 0;           // Used to fill the RAM
        SELINB = 1;
        CurrentMode = FunctionMode;
        break;
      case FreqCntrMode4:      // Used in the FC Mode. Input divided by 4
        SELINA = 1;
        SELINB = 0;
        CurrentMode = FreqCntrMode4;
        break;
      case FreqCntrMode1:       // Used in the FC Mode. Input direct
        SELINA = 0;
        SELINB = 0;
        CurrentMode = FreqCntrMode1;
        break;
    }
  }
 
 // This routine is used in the VVR Mode
 // It scans the Volts (SW2) and Tenths (SW3) switches and sends the resulting
 // output to the D/A converter
 
 void GetVVRSwitchSettings()
 {
   SetInputMode(PICDirectMode);
   
   while (1)
   {
     SelectedMode = ReadModeSwitch();

     if (SelectedMode != CurrentMode)
       break;
       
     SwitchReturn = SetAndReadAD(5, 1, 1  );  // Volts switch SW3
     delay_ms(200);
     Lcd_Cmd(_LCD_CLEAR);
     delay_ms(100);
     {
       Ref250 = 1;
       HalfStep = 0;

       if(SwitchReturn < (float).25)   // 0 volts
       {
         PORTDSet = 0;
       }
       else if(SwitchReturn < (float).75)   // 1 volt
       {
         PORTDSet = 25;
       }
        else if(SwitchReturn < (float)1.25)  // 2 volts
       {
         PORTDSet = 50;
       }
        else if(SwitchReturn < (float)1.75)   // 3 volts
       {
         PORTDSet = 75;
       }
        else if(SwitchReturn < (float)2.25)   // 4 volts
       {
         PORTDSet = 100;
       }
        else if(SwitchReturn < (float)2.75)    // 5 volts
       {
         PORTDSet = 125;
       }
        else if(SwitchReturn < (float)3.25)   // 6 volts
       {
         PORTDSet = 150;
       }
        else if(SwitchReturn < (float)3.75)    // 7 volts
       {
         PORTDSet = 175;
       }
        else if(SwitchReturn < (float)4.25)     // 8 volts
       {
         PORTDSet = 200;
       }
        else if(SwitchReturn < (float) 4.75)    // 9 volts
       {
         PORTDSet = 225;
       }
       else
       {
          PORTDSet = 250;                           // 10 volts
        }
     }

     // Now read temths volts

    if(SwitchReturn < (float) 4.75) // If integer switch set less than 10 volts
    {
     SwitchReturn = SetAndReadAD(6, 2, 1);    // Tenths switch SW3
     {
       if(SwitchReturn < (float).24)   // 0 volts
       {
       //  PORTDSet = 0;
       ByteToStr(PORTD, LCDString);
       WriteLCD(2,12 , LCDString);
       }
       else if(SwitchReturn < (float).75)   // 0.1 volt
       {
         PORTDSet += 3;
         HalfStep = 1;
        ;
       }
        else if(SwitchReturn < (float)1.4)  // 0.2 volts
       {
         PORTDSet += 5;
       }
        else if(SwitchReturn < (float)1.96)   // 0.3 volts
       {
         PORTDSet += 8;
         HalfStep = 1;
       }
        else if(SwitchReturn < (float)2.50)   // 0.4 volts
       {
         PORTDSet += 10;
       }
        else if(SwitchReturn < (float)3.05)    // 0.5 volts
       {
         PORTDSet += 13;
         HalfStep = 1;
       }
        else if(SwitchReturn < (float)3.61)   // 0.6 volts
       {
         PORTDSet += 15;
       }
        else if(SwitchReturn < (float)4.15)    // 0.7 volts
       {
         PORTDSet += 18;
         HalfStep = 1;
       }
        else if(SwitchReturn < (float)4.72)     // 0.8 volts
       {
         PORTDSet += 20;
       }
        else    // 0.9 volts
       {
         PORTDSet += 23;
         HalfStep = 1;
       }
     }

      PORTD = PORTDSet;
      ByteToStr(PORTDSet, LCDString);
      WriteLCD(2,12 , LCDString);
      delay_ms(500);
    }
     PORTD = PORTDSet;
      ByteToStr(PORTDSet, LCDString);
      WriteLCD(2,12 , LCDString);
      delay_ms(500);
    }

    }

/* This is the main program loop. It looks at the front panel MODE
   switch and sees if it differs from the current routine being processed.
   If it differs, it's stored in the Current Function register and the
   appropriate new mode routine is called.
   
   When a mode routine is called it continiously checks the MODE switch and
   exits to the main loop if a new routine is selected */

void EndRAMLoad ()
{
  SetInputMode(FunctionMode);   // Set to HS Clock
  TRISD = 0b11111111;       // Set to input mode so RAM can control the buss

  U4WE_ = 1;      // Set the RAM for normal read operation
  U4OE_ = 0;
}

void InitRAMLoad()
{
   ADCON0 = 0b00001010;   // Sets AN0 thru AN7 as A/D and Vdd as reference
   ADCON1 = 0b10111100;    // Right justified, Acg time and A/D clock
   TRISA = 0b00101111;     // Set A/D direction
   TRISE = 0b00000111;

   TRISB = 0b11000000;
   TRISC = 0b00000001;

   PICCLK = 1;
   U4OE_ = 1;             // Set the RAM for load operation
   U4WE_ = 1;
   TRISD = 0b00000000;

   U3MR_ = 1;
   U3MR_ = 0;      // Reset the RAM Addressing Counter
   U3MR_ = 1;

   SetInputMode(PICDirectMode);

   for(i=0; i<=255; i++)    // Zero out the RAM
   {
      PORTD = 0;
      WriteRAM();
      OnePulse();
   }

    U3MR_ = 1;
    U3MR_ = 0;      // Reset the RAM Addressing Counter
    U3MR_ = 1;

   HalfStep =0;
   Ref250 =0;
}

void RAMFillSign ()
{
   InitRAMLoad();

   if (CurrentBits == 64)    // Put 1 copy of sine wave (4 quadrants)
  {
    for(i = 0; i <= 63; i++)
    {
      PORTD = SinWave64[i] + 127;
      WriteRAM();
      OnePulse();
    }

    for(i = 0; i <= 63; i++)
    {
      PORTD = SinWave64[63 - i] + 127;
      WriteRAM();
      OnePulse();
    }

    for(i = 0; i <= 63; i++)
    {
      PORTD = - SinWave64[i] - 127;
      WriteRAM();
      OnePulse();
    }

    for(i = 0; i <= 63; i++)
    {
      PORTD = - SinWave64 [63 - i] - 127;
      WriteRAM();
      OnePulse();
    }
  }

  else if (CurrentBits == 32)      // Put in 2 copies of sine wave
  {
    for(i = 0; i <= 31; i++)
    {
      PORTD = SinWave64[2 * i] + 127;
      WriteRAM();
      OnePulse();
    }

    for(i = 0; i <= 31; i++)
    {
      PORTD = SinWave64[63 - 2 * i] + 127;
      WriteRAM();
      OnePulse();
    }

    for(i = 0; i <= 31; i++)
    {
      PORTD = - SinWave64[2 * i] - 127;
      WriteRAM();
      OnePulse();
    }

    for(i = 0; i <= 31; i++)
    {
      PORTD = - SinWave64 [63 - 2 * i] - 127;
      WriteRAM();
      OnePulse();
    }
    
    for(i = 0; i <= 31; i++)    // second copy
    {
      PORTD = SinWave64[2 * i] + 127;
      WriteRAM();
      OnePulse();
    }

    for(i = 0; i <= 31; i++)
    {
      PORTD = SinWave64[63 - 2 * i] + 127;
      WriteRAM();
      OnePulse();
    }

    for(i = 0; i <= 31; i++)
    {
      PORTD = - SinWave64[2 * i] - 127;
      WriteRAM();
      OnePulse();
    }

    for(i = 0; i <= 31; i++)
    {
      PORTD = - SinWave64 [63 - 2 * i] - 127;
      WriteRAM();
      OnePulse();
    }
   }

    else if (CurrentBits ==16)     // Repeats the waveform 4 times
    {
      for(i = 0; i <= 15; i++)
      {
        PORTD = SinWave64[4 * i] + 127;
        WriteRAM();
        OnePulse();
      }

      for(i = 0; i <= 15; i++)
      {
        PORTD = SinWave64[63 - 4 * i] + 127;
        WriteRAM();
        OnePulse();
      }

      for(i = 0; i <= 15; i++)
      {
        PORTD = - SinWave64[4 * i] - 127;
        WriteRAM();
        OnePulse();
      }

      for(i = 0; i <= 15; i++)
      {
        PORTD = - SinWave64 [63 - 4 * i] - 127;
        WriteRAM();
        OnePulse();
      }
      
      for(i = 0; i <= 15; i++)     //Second copy
      {
        PORTD = SinWave64[4 * i] + 127;
        WriteRAM();
        OnePulse();
      }

      for(i = 0; i <= 15; i++)
      {
        PORTD = SinWave64[63 - 4 * i] + 127;
        WriteRAM();
        OnePulse();
      }

      for(i = 0; i <= 15; i++)
      {
        PORTD = - SinWave64[4 * i] - 127;
        WriteRAM();
        OnePulse();
      }

      for(i = 0; i <= 15; i++)
      {
        PORTD = - SinWave64 [63 - 4 * i] - 127;
        WriteRAM();
        OnePulse();
      }
      
      for(i = 0; i <= 15; i++)        // Third copy
      {
        PORTD = SinWave64[4 * i] + 127;
        WriteRAM();
        OnePulse();
      }

      for(i = 0; i <= 15; i++)
      {
        PORTD = SinWave64[63 - 4 * i] + 127;
        WriteRAM();
        OnePulse();
      }

      for(i = 0; i <= 15; i++)
      {
        PORTD = - SinWave64[4 * i] - 127;
        WriteRAM();
        OnePulse();
      }

      for(i = 0; i <= 15; i++)
      {
        PORTD = - SinWave64 [63 - 4 * i] - 127;
        WriteRAM();
        OnePulse();
      }

      for(i = 0; i <= 15; i++)        // Fourth copy
      {
       PORTD = SinWave64[4 * i] + 127;
        WriteRAM();
        OnePulse();
      }

      for(i = 0; i <= 15; i++)
      {
        PORTD = SinWave64[63 - 4 * i] + 127;
        WriteRAM();
        OnePulse();
      }

      for(i = 0; i <= 15; i++)
      {
        PORTD = - SinWave64[4 * i] - 127;
        WriteRAM();
        OnePulse();
      }

      for(i = 0; i <= 15; i++)
      {
        PORTD = - SinWave64 [63 - 4 * i] - 127;
        WriteRAM();
        OnePulse();
      }
   }
   
   EndRAMLoad();
}


void RAMFillSquare ()
{
   InitRAMLoad();
   
  if (CurrentBits == 64)
  {
    for(i = 0; i <= 127; i++)
    {
      PORTD = 0;
      WriteRAM();
      OnePulse();
    }
    for(i = 0; i <= 127; i++)
    {
      PORTD = 255;
      WriteRAM();
      OnePulse();
    }
  }
  if (CurrentBits == 32)
  {
    for(i = 0; i <= 63; i++)
    {
      PORTD = 0;
      WriteRAM();
      OnePulse();
    }
    for(i = 0; i <= 63; i++)
    {
      PORTD = 255;
      WriteRAM();
      OnePulse();
    }
    for(i = 0; i <= 63; i++)
    {
      PORTD = 0;
      WriteRAM();
      OnePulse();
    }
    for(i = 0; i <= 63; i++)
    {
      PORTD = 255;
      WriteRAM();
      OnePulse();
    }
  }
 if (CurrentBits == 16)
  {
    for(i = 0; i <= 31; i++)
    {
      PORTD = 0;
      WriteRAM();
      OnePulse();
    }
    for(i = 0; i <= 31; i++)
    {
      PORTD = 255;
      WriteRAM();
      OnePulse();
    }
    for(i = 0; i <= 31; i++)
    {
      PORTD = 0;
      WriteRAM();
      OnePulse();
    }
    for(i = 0; i <= 31; i++)
    {
      PORTD = 255;
      WriteRAM();
      OnePulse();
    }
    for(i = 0; i <= 31; i++)
    {
      PORTD = 0;
      WriteRAM();
      OnePulse();
    }
    for(i = 0; i <= 31; i++)
    {
      PORTD = 255;
      WriteRAM();
      OnePulse();
    }
    for(i = 0; i <= 31; i++)
    {
      PORTD = 0;
      WriteRAM();
      OnePulse();
    }
    for(i = 0; i <= 31; i++)
    {
      PORTD = 255;
      WriteRAM();
      OnePulse();
    }
  }
  EndRAMLoad();
}

void RAMFillSaw ()
{
   InitRAMLoad();
   
  if (CurrentBits == 64)
  {
   for(i = 0; i <= 127; i++)
    {
      PORTD = i;
      WriteRAM();
      OnePulse();
    }
    for(i = 0; i <= 127; i++)
    {
      PORTD = 127 - i;
      WriteRAM();
      OnePulse();
    }
   }

 else if (CurrentBits == 32)
  {
   for(i = 0; i <= 63; i++)
    {
      PORTD = 2 * i;
      WriteRAM();
      OnePulse();
    }
    for(i = 0; i <= 63; i++)
    {
      PORTD = 127 - 2 * i;
      WriteRAM();
      OnePulse();
    }
    
    for(i = 0; i <= 63; i++)
    {
      PORTD = 2 * i;
      WriteRAM();
      OnePulse();
    }
    for(i = 0; i <= 63; i++)
    {
      PORTD = 127 - 2 * i;
      WriteRAM();
      OnePulse();
    }
   }
   
   else if (CurrentBits == 16)
  {
    for(i = 0; i <= 31; i++)
    {
      PORTD = 4 * i;
      WriteRAM();
      OnePulse();
    }
    for(i = 0; i <= 31; i++)
    {
      PORTD = 127 - 2 * i;
      WriteRAM();
      OnePulse();
    }

    for(i = 0; i <= 31; i++)
    {
      PORTD = 4 * i;
      WriteRAM();
      OnePulse();
    }
    for(i = 0; i <= 31; i++)
    {
      PORTD = 127 - 4 * i;
      WriteRAM();
      OnePulse();
    }
    
    for(i = 0; i <= 31; i++)
    {
      PORTD = 4 * i;
      WriteRAM();
      OnePulse();
    }
    for(i = 0; i <= 31; i++)
    {
      PORTD = 127 - 42 * i;
      WriteRAM();
      OnePulse();
    }

    for(i = 0; i <= 31; i++)
    {
      PORTD = 4 * i;
      WriteRAM();
      OnePulse();
    }
    for(i = 0; i <= 31; i++)
    {
      PORTD = 127 - 4 * i;
      WriteRAM();
      OnePulse();
    }
   }
   
   EndRAMLoad();
}

void RAMFillRamp ()
{
   InitRAMLoad();
   
  if (CurrentBits == 64)
  {
    for(i = 0; i <= 255; i++)
    {
      PORTD = i;
      WriteRAM();
      OnePulse();
    }
  }
  else if (CurrentBits == 32)
  {
    for(i = 0; i <= 127; i++)
    {
      PORTD = 2 * i;
      WriteRAM();
      OnePulse();
    }

     for(i = 0; i <= 127; i++)
    {
      PORTD = 2 * i;
      WriteRAM();
      OnePulse();
    }
  }
  
  else if (CurrentBits == 16)
  {
    for(i = 0; i <= 63; i++)
    {
      PORTD = 4 * i;
      WriteRAM();
      OnePulse();
    }

    for(i = 0; i <= 63; i++)
    {
      PORTD = 4 * i;
      WriteRAM();
      OnePulse();
    }
    
    for(i = 0; i <= 63; i++)
    {
      PORTD = 4 * i;
      WriteRAM();
      OnePulse();
    }
    for(i = 0; i <= 63; i++)
    {
      PORTD = 4 * i;
      WriteRAM();
      OnePulse();
    }
  }
  
  EndRAMLoad();
}

void RAMFillPulse ()
{
  InitRAMLoad();
   
  if (CurrentBits == 64)
  {
   for(i = 0; i <= 245; i++)
    {
      PORTD = 0;
      WriteRAM();
      OnePulse();
    }
    for(i = 0; i <= 10; i++)
    {
      PORTD = 255;
      WriteRAM();
      OnePulse();
    }
   }

  else if (CurrentBits == 32)
  {
   for(i = 0; i <= 123; i++)
    {
      PORTD = 0;
      WriteRAM();
      OnePulse();
    }
    for(i = 0; i <= 5; i++)
    {
      PORTD = 255;
      WriteRAM();
      OnePulse();
    }
   }
   else if (CurrentBits == 16)
   {
     for(i = 0; i <= 61; i++)
     {
       PORTD = 0;
       WriteRAM();
       OnePulse();
     }
      for(i = 0; i <= 3; i++)
      {
        PORTD = 255;
        WriteRAM();
        OnePulse();
      }
      for(i = 0; i <= 61; i++)
     {
       PORTD = 0;
       WriteRAM();
       OnePulse();
     }
      for(i = 0; i <= 3; i++)
      {
        PORTD = 255;
        WriteRAM();
        OnePulse();
      }
      for(i = 0; i <= 61; i++)
     {
       PORTD = 0;
       WriteRAM();
       OnePulse();
     }
      for(i = 0; i <= 3; i++)
      {
        PORTD = 255;
        WriteRAM();
        OnePulse();
      }
      for(i = 0; i <= 61; i++)
     {
       PORTD = 0;
       WriteRAM();
       OnePulse();
     }
      for(i = 0; i <= 3; i++)
      {
        PORTD = 255;
        WriteRAM();
        OnePulse();
      }
   }
    EndRAMLoad();
}

void RAMFillF1 ()    // User defined
{
}

void RAMFillF2 ()    // User defined
{
}

void RAMFillF3 ()    // User defined
{
}

void DoFreqCounter ()
{
  while (1)
  {
    SetInputMode(FreqCntrMode1);
    SelectedMode = ReadModeSwitch();

     if (SelectedMode != CurrentMode)
       break;

    ReadFrequency(1);

   if (ulFrequency <= 10000)  // Actually 100,000 - do a period measurement
   {
      if (ulFrequency < 1)
        ulFrequency = 1;

      ulFrequency *= 5;
      ReadPeriod(ulFrequency);

      FloatToStr(fFrequency, LCDString);
      strcat(LCDString, "  P");
    }
    else
    {
      ReadFrequency(10);

      LongToStr(ulFrequency, LCDString);
      DeleteLeadingSpaces(LCDString);
      strcat(LCDString, "  F");
    }

      Lcd_Cmd(_LCD_CLEAR);
      delay_ms(20);
      WriteLCD(2,1, LCDString);

   }
}


void main()
{
  TRISC = 0b00000001;

  ADCON0 = 0b10000001;    // Sets AN0 thru AN7 as A/D, A/D clock at Fosc = 64
  ADCON1 = 0b11111000;    // Right justified, A/D clock, Exteral source ref

  TRISA = 0b11101111;     // Set A/D direction  Port A
  TRISB = 0b11000000;     // LCD control
  TRISE = 0b00000111;     // Set A/D direction  Port E  '

   TRISD = 0b00000000;
   U4OE_ = 1;
   HalfStep =0;
   Ref250 =0;
   PORTD = 255;

   Lcd_Init();                       // Initialize Lcd
   Lcd_Cmd(_LCD_CLEAR);              // Clear display
   Lcd_Cmd(_LCD_CURSOR_OFF);

   U3MR_ = 1;
   INTCON = 0;  // Disable Interrupts
   CurrentMode = 0;

   while (1) // This forms a continuous loop checking the MODE and Bits switches
   {
     delay_ms(50);
     Lcd_Cmd(_LCD_CLEAR);
     SelectedBits = SelectBits();
     SelectedMode = ReadModeSwitch();

     delay_ms(100);

     if ((SelectedMode != CurrentMode) || (SelectedBits != CurrentBits))
     {
       CurrentMode = SelectedMode;
       CurrentBits = SelectedBits;

       if (SelectedMode == FCMode)
       {
         SetInputMode(FreqCntrMode1);

         DoFreqCounter ();
         
         while (1)
         {
           SelectedBits = SelectBits();
           SelectedMode = ReadModeSwitch();

           if ((SelectedMode != CurrentMode) || (SelectedBits != CurrentBits))
           break;      // go back to main loop
         }
       }

       else if (SelectedMode == VVRMode)
       {
          GetVVRSwitchSettings ();
          
         while (1)
         {
           SelectedBits = SelectBits();
           SelectedMode = ReadModeSwitch();

           if ((SelectedMode != CurrentMode) || (SelectedBits != CurrentBits))
           break;      // go back to main loop
         }
       }

       else if (SelectedMode == SineMode)
       {
         RAMFillSign ();

         while (1)
         {
           SelectedBits = SelectBits();
           SelectedMode = ReadModeSwitch();
           
           if ((SelectedMode != CurrentMode) || (SelectedBits != CurrentBits))
           break;      // go back to main loop
         }
       }

       else if (SelectedMode == SquareMode)
       {
         RAMFillSquare ();
         
         while (1)
         {
           SelectedBits = SelectBits();
           SelectedMode = ReadModeSwitch();

           if ((SelectedMode != CurrentMode) || (SelectedBits != CurrentBits))
           break;      // go back to main loop
         }
       }

       else if (SelectedMode == RampMode)
       {
         RAMFillRamp ();
         
         while (1)
         {
           SelectedBits = SelectBits();
           SelectedMode = ReadModeSwitch();

           if ((SelectedMode != CurrentMode) || (SelectedBits != CurrentBits))
           break;      // go back to main loop
         }
       }

       else if (SelectedMode == SawMode)
       {
         RAMFillSaw ();
         
         while (1)
         {
           SelectedBits = SelectBits();
           SelectedMode = ReadModeSwitch();

           if ((SelectedMode != CurrentMode) || (SelectedBits != CurrentBits))
           break;      // go back to main loop
         }
       }

       else if (SelectedMode == PulseMode)
       {
         RAMFillPulse ();
         
         while (1)
         {
           SelectedBits = SelectBits();
           SelectedMode = ReadModeSwitch();

           if ((SelectedMode != CurrentMode) || (SelectedBits != CurrentBits))
           break;      // go back to main loop
         }
       }

       else if (SelectedMode == F1Mode) // User defined
       {
  //       DoF1Mode ();  // User defined
       }

       else if (SelectedMode == F2Mode)
       {
  //       DoF2Mode (); // User defined
       }

       else if (SelectedMode == F3Mode)
       {
    //     DoF3Mode (); // User defined
       }
     }
   } // while (1)
 }
